#include adovfp.h

* Business-layer class.

define class BusinessClass as COMBase of COMBase.fxp
	protected cDataProgID, ;	&& The ProgID for the data object this class works with
		oData, ;				&& A reference to a DataClass object
		cServer					&& The name of the server for DCOM objects
	cDataProgID = ''
	oData       = .NULL.
	cServer     = ''
	Name        = 'BusinessClass'

	procedure Init(tcProgID as String)
		local lcDataProgID
		with This

* Although the Init method of COM objects can't receive parameters, if this
* class is instantiated as a VFP class (usually for debugging purposes),
* we'll accept a ProgID for our data class.

			lcDataProgID = iif(vartype(tcProgID) = 'C' and ;
				not empty(tcProgID), tcProgID, '')

* Instantiate the data object we're working with and set any properties we
* need.

			do case
				case not empty(lcDataProgID)
					.oData = createobject(lcDataProgID)
				case empty(.cServer)
					.oData = createobject(.cDataProgID)
				otherwise
					.oData = createobjectex(.cDataProgID, .cServer)
			endcase
			.SetDataProperties()
		endwith
	endproc

* Nuke the data object when this object is destroyed.

	procedure Destroy
		This.oData = .NULL.
	endproc

* Delete the current record.

	procedure Delete(toRS as Object) as Boolean
		local llReturn
		with This
			llReturn = .oData.Delete(toRS)
			if not llReturn
				.ErrorMessage  = .oData.ErrorMessage
				.ErrorOccurred = .T.
			endif not llReturn
		endwith
		return llReturn
	endproc

* Save the specified recordset.

	procedure Save(toRS as Object) as Boolean
		local llReturn
		with This
			.ResetError()
			do case

* If Validate the recordset. If it failed, return .F.

				case not inlist(toRS.Status, adRecDeleted, adRecDBDeleted) and ;
					not .ValidateData(toRS)
					llReturn = .F.

* Have the data object save the recordset. If it failed, get the error message
* and return .F.

				case not .oData.Save(toRS)
					.ErrorMessage  = .oData.ErrorMessage
					.ErrorOccurred = .T.
					llReturn       = .F.

* Everything worked, so return .T.

				otherwise
					llReturn = not .ErrorOccurred
			endcase
		endwith
		return llReturn
	endproc

* Requeries the recordset.

	procedure Requery(toRS as Object) as Boolean
		local llReturn
		with This
			.ResetError()
			llReturn = .oData.Requery(toRS)
			if not llReturn
				.ErrorMessage = .oData.ErrorMessage
			endif not llReturn
		endwith
		return llReturn
	endproc

* Ensure the specified recordset has valid data. This is an abstract method in
* this class.

	protected procedure ValidateData(toRS)
	endproc

* Set properties of the data object. This is an abstract method in this class.

	protected procedure SetDataProperties
	endproc
enddefine

* Data-layer class.

define class DataClass as COMBase of COMBase.fxp olepublic
	protected oConnection		&& A reference to an ADO Connection object
	oConnection       = .NULL.
	cDSN              = ''		&& The DSN to use; leave it blank if no DSN is
								&& used and specify cProvider and
								&& cConnectionString instead
	cProvider         = ''		&& The OLE DB provider to use
	cConnectionString = ''		&& The connection string
	Name = 'DataClass'

* Create an ADO Connection object when this object is created.

	procedure Init
		This.oConnection = createobject('ADODB.Connection')
	endproc

* When this object is destroyed, close the connection if necessary and nuke the
* connection object.

	procedure Destroy
		with This
			if .oConnection.State = adStateOpen
				.oConnection.Close()
			endif .oConnection.State = adStateOpen
			.oConnection = .NULL.
		endwith
	endproc

* Open the passed recordset.

	procedure GetRecordSet(toRS as Object, tcCommand as String) as Boolean
		local llReturn
		with This

* Open the connection if necessary. If we succeeded, carry on.

			if .OpenConnection()
				.ResetError()

* If the recordset is already open, close it.

				if toRS.State = adStateOpen
					toRS.CancelBatch()
					toRS.Close()
				endif toRS.State = adStateOpen

* Set properties of the recordset, then open it.

				toRS.ActiveConnection = .oConnection
				.SetRecordSetProperties(toRS)
				toRS.Open(tcCommand)

* Disconnect the recordset.

				toRS.ActiveConnection = .NULL.
				llReturn = toRS.State = adStateOpen

* We couldn't open the connection, so return .F.

			else
				llReturn = .F.
			endif .OpenConnection()
		endwith
		return llReturn
	endproc

* Delete the current record.

	procedure Delete(toRS as Object) as Boolean
		with toRS
			if This.OpenConnection()
				.ActiveConnection = This.oConnection
				.Delete()
				.ActiveConnection = .NULL.
				llReturn = not This.ErrorOccurred
			else
				llReturn = .F.
			endif This.OpenConnection()
		endwith
		return llReturn
	endproc

* Save the specified recordset.

	procedure Save(toRS as Object) as Boolean
		local llReturn
		with This

* Open the connection if necessary. If we succeeded, reset the error flag and
* determine how to save the recordset.

			if .OpenConnection()
				.ResetError()
				do case

* If we have a batch optimistic recordset, start a transaction, do a batch
* update, and either commit or rollback as necessary.

					case toRS.LockType = adLockBatchOptimistic
						.oConnection.BeginTrans()
						toRS.ActiveConnection = .oConnection
						toRS.UpdateBatch()
						if .ErrorOccurred
							.oConnection.RollbackTrans()
							toRS.CancelBatch()
						else
							.oConnection.CommitTrans()
						endif .ErrorOccurred
						toRS.ActiveConnection = .NULL.

* If we have an optimistic recordset, update it.

					case toRS.LockType = adLockOptimistic
						toRS.ActiveConnection = .oConnection
						toRS.Update()
						if .ErrorOccurred
							toRS.Cancel()
						endif .ErrorOccurred
						toRS.ActiveConnection = .NULL.

* We don't have an updatable recordset, so raise an error.

					otherwise
						error 'The recordset is not updatable.'
				endcase
				llReturn = not .ErrorOccurred
			else
				llReturn = .F.
			endif .OpenConnection()
		endwith
		return llReturn
	endproc

* Requerys the recordset.

	procedure Requery(toRS as Object) as Boolean
		local llReturn
		with This

* Open the connection if necessary. If we succeeded, reset the error flag.
* Reconnect the recordset, cancel any changes that've been made, and requery
* it, then disconnect it again.

			if .OpenConnection()
				.ResetError()
				toRS.ActiveConnection = .oConnection
				toRS.CancelBatch()
				toRS.Requery()
				toRS.ActiveConnection = .NULL.
				llReturn = not .ErrorOccurred
			else
				llReturn = .F.
			endif .OpenConnection()
		endwith
		return llReturn
	endproc

* Set properties of the oConnection object.

	protected procedure SetConnectionProperties
		with This
			if empty(.cDSN)
				.oConnection.Provider         = .cProvider
				.oConnection.ConnectionString = .cConnectionString
			endif empty(.cDSN)
		endwith
	endproc

* Open the connection object.

	protected procedure OpenConnection
		local llReturn
		with This

* If the connection isn't already open, open it.

			if .oConnection.State <> adStateOpen

* Set properties of the connection object first.

				.SetConnectionProperties()

* If we aren't using a DSN, the Provider and ConnectionString properties of the
* connection object have been set, so open the connection. Otherwise, use the
* DSN to open the connection.

				if empty(.cDSN)
					.oConnection.Open()
				else
					.oConnection.Open(.cDSN)
				endif empty(.cDSN)

* Return .T. if the connection was opened.

				llReturn = .oConnection.State = adStateOpen
			else
				llReturn = .T.
			endif .oConnection.State <> adStateOpen
		endwith
		return llReturn
	endproc

* Set properties of the specified recordset.

	protected procedure SetRecordSetProperties(toRS)
		toRS.CursorLocation = adUseClient
	endproc
enddefine
